package tests;

import entities.noorderbyannotation.Department;
import entities.noorderbyannotation.Employee;
import entities.noorderbyannotation.EmployeeComparator;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import testhelpers.assertionhelpers.AssertionHelper;

public class TestNoOrderByAnnotation {
    
    private String testID;
    private EntityManagerFactory emf;
    
    private AssertionHelper assertionHelper;
    
    private static long DEPT_ID777 = 777l;
    private static String DEPT_NAME777 = "SevenSeventySeven";
    private static long EMP_ID_FIRST = 34l;
    private static String EMP_CODE_FIRST = "ThirtyFourEL777";
    
    private static long DEPT_ID888 = 888l;
    private static String DEPT_NAME888 = "EightEightyEight";
    private static long EMP_ID_SECOND = 35l;
    private static String EMP_CODE_SECOND = "ThirtyFiveEL888";
    
    private static long DEPT_ID999 = 999l;
    private static String DEPT_NAME999 = "NineNinetyNine";
    private static long EMP_ID_THIRD = 36l;
    private static String EMP_CODE_THIRD = "ThirtySixEL999";
    
    private Employee empFirst;
    private Employee empSecond;
    private Employee empThird;

    private List<Employee> employeeList;
    
    
    public TestNoOrderByAnnotation(String testSuiteID, AssertionHelper assertionHelper) {
        this.testID = testSuiteID + "NoOrderByAnnotation";
        this.assertionHelper = assertionHelper;
    }
    
    public void test() {
        emf = Persistence.createEntityManagerFactory("mapkey-orderby");
        persistRelationship();
        assertRelationshipRetrievedInRightOrder();
    }
    
    private void persistRelationship() {
        try {
            Department dept = createDepartment(DEPT_ID777, DEPT_NAME777);
            empFirst = createEmployee(EMP_ID_FIRST, EMP_CODE_FIRST);
            empSecond = createEmployee(EMP_ID_SECOND, EMP_CODE_SECOND);
            empThird = createEmployee(EMP_ID_THIRD, EMP_CODE_THIRD);
            
            // Not adding employees in Order of id
            employeeList = new ArrayList<Employee>();
            employeeList.add(empFirst);
            employeeList.add(empThird);
            employeeList.add(empSecond);
            
            dept.setEmployees(employeeList);
            empFirst.setDepartment(dept);
            empSecond.setDepartment(dept);
            empThird.setDepartment(dept);
            
            EntityManager em = emf.createEntityManager();
            em.getTransaction().begin();
            em.persist(dept);
            em.getTransaction().commit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    private void assertRelationshipRetrievedInRightOrder() {
        EntityManager em = emf.createEntityManager();
        
        Collections.sort(employeeList, new EmployeeComparator());
        
        Department dept = em.find(Department.class, DEPT_ID777);
        List<Employee> actualList = dept.getEmployees();
        
        if (employeeList.equals(actualList))
            assertionHelper.reportPass(testID);
        else{
            assertionHelper.reportFail(testID, "The expected Employee List is not equal to the actual List read from the DB");
            printBothLists(employeeList, actualList);
        }
    }
    
    private Employee createEmployee(long id, String code) {
        Employee emp = new Employee();
        emp.setId(id);
        emp.setCode(code);
        return emp;
    }
    
    private Department createDepartment(long id, String name) {
        Department dept = new Department();
        dept.setId(id);
        dept.setName(name);
        return dept;
    }

    private void printBothLists(List<Employee> employeeList, List actualList) {
        System.out.println(testID + " Expected List - ");
        printList(employeeList);
        System.out.println(testID + " Actual List (from DB) - ");
        printList(actualList);
    }

    private void printList(List<Employee> employeeList) {
        for(Employee emp : employeeList){
            System.out.println("Emp ID - " + emp.getId());
        }
    }
    
}
