package tests;

import entities.overridemapkey.TheatreCompany;
import entities.overridemapkey.TheatreLocation;
import java.util.HashMap;
import java.util.Map;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import testhelpers.assertionhelpers.AssertionHelper;

public class TestOverrideMapKey {
    
    private String testID;
    private EntityManagerFactory emf;
    
    private AssertionHelper assertionHelper;
    
    private static long REGAL_ID = 676l;
    private static String REGAL_NAME = "Regal";
    private static long KNOXVILLE_ID = 42l;
    private static String KNOXVILLE_CODE = "KXTN";
    private static long IOWA_CITY_ID = 47l;
    private static String IOWA_CITY_CODE = "ICIO";
    
    public TestOverrideMapKey(String testSuiteID, AssertionHelper assertionHelper) {
        this.testID = testSuiteID + "OverrideMapKey";
        this.assertionHelper = assertionHelper;
    }
    
    public void test() {
        emf = Persistence.createEntityManagerFactory("mapkey-orderby");
        persistRelationship();
        assertCorrectMapKeyUsedOnCollection();
    }
    
    private void persistRelationship() {
        try {
            TheatreCompany regal = createTheatreCompany(REGAL_ID, REGAL_NAME);
            TheatreLocation knoxville = createTheatreLocation(KNOXVILLE_ID, KNOXVILLE_CODE);
            TheatreLocation iowacity = createTheatreLocation(IOWA_CITY_ID, IOWA_CITY_CODE);
            
            Map<String, TheatreLocation> regalLocations = new HashMap<String, TheatreLocation>();
            regalLocations.put(KNOXVILLE_CODE, knoxville);
            regalLocations.put(IOWA_CITY_CODE, iowacity);
            
            regal.setLocations(regalLocations);
            knoxville.addCompany(regal);
            iowacity.addCompany(regal);
            
            EntityManager em = emf.createEntityManager();
            em.getTransaction().begin();
            em.persist(regal);
            em.getTransaction().commit();
            em.close();
            
        } catch (Exception e) {
            assertionHelper.reportFail(testID, "Expected Cascade Persist to have applied. Persisting company must persist location");
            e.printStackTrace();
        }
    }
    
    private void assertCorrectMapKeyUsedOnCollection() {
        try {
            EntityManager em = emf.createEntityManager();
            
            TheatreCompany retrievedRegal = em.find(TheatreCompany.class, REGAL_ID);
            Map<String, TheatreLocation> regalLocations = retrievedRegal.getLocations();

            
            // CODE as the key !!
            
            if ((regalLocations.get(KNOXVILLE_CODE).getId() == KNOXVILLE_ID) &&
                    (regalLocations.get(IOWA_CITY_CODE).getId() == IOWA_CITY_ID) ) {
                assertionHelper.reportPass(testID);
            } else
                assertionHelper.reportFail(testID, "Expected to read the relationship as a Map with keys - " + KNOXVILLE_CODE + " and " + IOWA_CITY_CODE + 
                        "; Actual Locations Found - " + regalLocations.get(KNOXVILLE_CODE).getId() + " and " + regalLocations.get(IOWA_CITY_CODE).getId());
            
            // ID as the key !!
            
//            if ((regalLocations.get(KNOXVILLE_ID).getCode().equals(KNOXVILLE_CODE)) &&
//                    (regalLocations.get(IOWA_CITY_ID).getCode().equals(IOWA_CITY_CODE)) ) {
//                assertionHelper.reportPass(testID);
//            } else
//                assertionHelper.reportFail(testID, "Expected to read the relationship as a Map with keys - " + KNOXVILLE_ID + " and " + IOWA_CITY_ID + 
//                        "; Actual Locations Found - " + regalLocations.get(KNOXVILLE_ID).getCode() + " and " + regalLocations.get(IOWA_CITY_ID).getCode());
            
        } catch (Exception e) {
            assertionHelper.reportFailUnexpectedException(testID);
            e.printStackTrace();
        }
    }
    
    private TheatreLocation createTheatreLocation(long id, String code) {
        TheatreLocation loc = new TheatreLocation();
        loc.setId(id);
        loc.setCode(code);
        return loc;
    }
    
    private TheatreCompany createTheatreCompany(long id, String name) {
        TheatreCompany company = new TheatreCompany();
        company.setId(id);
        company.setName(name);
        return company;
    }
    
}
